"""Constants."""

CONFIG_DIR = ".slices"

SLICES_AI_CONFIG_CACHE_FILENAME = "cache_slices-ai-config.json"
DEFAULT_SLICES_AI_CONFIG_URL = "https://portal.slices-ri.eu/ai-service-config.json"
SLICES_AI_CONFIG_CACHE_TIMEOUT_MINUTES = 5
